"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseOptions = exports.parseStepSize = exports.DEFAULT_DRUNK_STEP = void 0;
const clamp_1 = __importDefault(require("./clamp"));
const Rand_1 = __importDefault(require("./Rand"));
exports.DEFAULT_DRUNK_STEP = 0.1;
const parseStepSize = (step) => typeof step !== 'undefined' ? (0, clamp_1.default)(step, 0, 1) : exports.DEFAULT_DRUNK_STEP;
exports.parseStepSize = parseStepSize;
const parseOptions = (opts) => {
    const _a = opts || {}, { step } = _a, restOpts = __rest(_a, ["step"]);
    const parsedStepSize = (0, exports.parseStepSize)(step);
    return Object.assign({ max: 1, min: 0, startsAt: undefined, step: parsedStepSize }, restOpts);
};
exports.parseOptions = parseOptions;
class Drunk {
    constructor(opts) {
        const { min, max, step, startsAt } = (0, exports.parseOptions)(opts);
        this._initialValue = new Rand_1.default({ min, max });
        this._step = new Rand_1.default({ min: -1, max: 1 });
        const initialValue = typeof startsAt !== 'undefined' ? startsAt : this._initialValue.value();
        this.state = {
            initialValue,
            max,
            min,
            step,
            value: initialValue,
        };
    }
    setRange(partialOpts) {
        const { max, min } = Object.assign({ min: this.state.min, max: this.state.max }, partialOpts);
        this._initialValue.setRange({ min, max });
        this.state = Object.assign(Object.assign({}, this.state), (min !== this.state.min || max !== this.state.max
            ? {
                initialValue: (0, clamp_1.default)(this._initialValue.value(), min, max),
                max,
                min,
                value: (0, clamp_1.default)(this.state.value, min, max),
            }
            : {
                max,
                min,
            }));
    }
    setStepSize(partialOpts) {
        const step = (0, exports.parseStepSize)(partialOpts.step);
        this.state = Object.assign(Object.assign({}, this.state), { step });
    }
    reset(opts) {
        const { min, max, startsAt, step } = (0, exports.parseOptions)(opts);
        this.setRange({ min, max });
        this.setStepSize({ step });
        const initialValue = typeof startsAt !== 'undefined' ? startsAt : this._initialValue.next();
        this.state = Object.assign(Object.assign({}, this.state), { initialValue, value: initialValue });
    }
    value() {
        return this.state.value;
    }
    next() {
        const { min, max, step, value } = this.state;
        const updates = (0, clamp_1.default)(value + max * this._step.next() * step, min, max);
        this.state.value = updates;
        return updates;
    }
}
exports.default = Drunk;
