"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDerivedStateFromOptions = exports.parseOptions = void 0;
const now_1 = __importDefault(require("../utils/now"));
const Scale_1 = __importDefault(require("./Scale"));
const parseOptions = (opts) => {
    return Object.assign({ from: 0, to: 1 }, opts);
};
exports.parseOptions = parseOptions;
const getDerivedStateFromOptions = (opts, prevState) => {
    const { from, to, duration } = Object.assign(Object.assign({}, prevState), opts);
    return Object.assign(Object.assign({}, prevState), { duration,
        from,
        to, totalElapsed: 0 });
};
exports.getDerivedStateFromOptions = getDerivedStateFromOptions;
class Env {
    constructor(opts) {
        const { from, to, duration } = (0, exports.parseOptions)(opts);
        this._interpolator = new Scale_1.default({
            from: {
                min: 0,
                max: duration,
            },
            to: {
                min: from,
                max: to,
            },
        });
        this.state = {
            duration,
            from,
            prev: (0, now_1.default)(),
            to,
            totalElapsed: 0,
            value: from,
        };
    }
    setDuration(duration) {
        const { to, totalElapsed } = this.state;
        this.state = Object.assign(Object.assign({}, this.state), (duration <= totalElapsed
            ? {
                duration,
                value: to,
            }
            : { duration }));
    }
    reset(opts) {
        const updates = (0, exports.getDerivedStateFromOptions)(opts, this.state);
        this.state = Object.assign(Object.assign({}, updates), { prev: (0, now_1.default)(), value: updates.from });
        this._interpolator.setRanges({
            from: {
                min: 0,
                max: updates.duration,
            },
            to: {
                min: updates.from,
                max: updates.to,
            },
        });
    }
    hasFinished() {
        return this.state.duration <= this.state.totalElapsed;
    }
    value() {
        const { to, value } = this.state;
        if (this.hasFinished()) {
            return to;
        }
        return value;
    }
    next() {
        if (this.hasFinished()) {
            return this.value();
        }
        const { from, prev, to, totalElapsed: prevTotalElapsed } = this.state;
        const curr = (0, now_1.default)();
        const tickInterval = curr - prev;
        const totalElapsed = prevTotalElapsed + tickInterval;
        const updates = this._interpolator.scale(totalElapsed);
        const value = from > to ? Math.min(updates, from) : Math.min(updates, to);
        this.state.prev = curr;
        this.state.totalElapsed = totalElapsed;
        this.state.value = value;
        return value;
    }
}
exports.default = Env;
