"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDerivedStateFromOptions = exports.parseOptions = void 0;
const clamp_1 = __importDefault(require("./clamp"));
const parseOptions = (opts) => {
    const { from, to } = Object.assign(Object.assign({}, opts), { from: Object.assign({ min: 0, max: 1 }, opts === null || opts === void 0 ? void 0 : opts.from), to: Object.assign({ min: 0, max: 1 }, opts === null || opts === void 0 ? void 0 : opts.to) });
    return {
        from,
        to,
    };
};
exports.parseOptions = parseOptions;
const getDerivedStateFromOptions = (opts, prevState) => {
    const { from, to } = opts;
    const updatedTo = Object.assign(Object.assign({}, prevState.to), to);
    return Object.assign(Object.assign({}, prevState), { from: Object.assign(Object.assign({}, prevState.from), from), to: updatedTo, value: (0, clamp_1.default)(prevState.value, updatedTo.min, updatedTo.max) });
};
exports.getDerivedStateFromOptions = getDerivedStateFromOptions;
class Scale {
    static scale(n, opts) {
        return new Scale(opts).scale(n);
    }
    constructor(opts) {
        const { from, to } = (0, exports.parseOptions)(opts);
        this.state = { from, to, value: undefined };
    }
    setRanges(opts) {
        this.state = (0, exports.getDerivedStateFromOptions)(opts, this.state);
    }
    reset(opts) {
        const { from, to } = (0, exports.parseOptions)(opts);
        this.state = { from, to, value: undefined };
    }
    value() {
        return this.state.value;
    }
    scale(n) {
        const { from, to } = this.state;
        const value = to.min +
            (((0, clamp_1.default)(n, from.min, from.max) - from.min) * (to.max - to.min)) /
                (from.max - from.min);
        this.state.value = value;
        return value;
    }
}
exports.default = Scale;
