"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SINE_PERIOD = void 0;
const Scale_1 = __importDefault(require("./Scale"));
const clamp_1 = __importDefault(require("./clamp"));
const now_1 = __importDefault(require("../utils/now"));
exports.SINE_PERIOD = Math.PI * 2 - 0.0001;
const getInitialState = (duration) => ({
    cycle: 0,
    duration,
    prev: (0, now_1.default)(),
    totalElapsed: 0,
    value: 0,
});
class Sine {
    constructor(opts) {
        const { duration } = opts;
        this._interpolator = new Scale_1.default({
            from: {
                min: 0,
                max: duration,
            },
            to: {
                min: 0,
                max: exports.SINE_PERIOD,
            },
        });
        this.state = getInitialState(duration);
    }
    setDuration(duration) {
        this.state = Object.assign(Object.assign({}, this.state), { duration });
    }
    reset(opts) {
        const { duration } = Object.assign(Object.assign({}, this.state), opts);
        this.state = getInitialState(duration);
    }
    value() {
        return this.state.value;
    }
    next() {
        const { cycle, duration, prev, totalElapsed: prevTotalElapsed, } = this.state;
        const curr = (0, now_1.default)();
        const tickInterval = curr - prev;
        const totalElapsed = prevTotalElapsed + tickInterval;
        const updates = this._interpolator.scale(totalElapsed);
        const value = (0, clamp_1.default)(Math.sin(updates), -1, 1);
        if (cycle >= duration) {
            this.state.cycle = 0;
        }
        else {
            this.state.cycle = cycle + tickInterval;
        }
        this.state.prev = curr;
        this.state.totalElapsed = totalElapsed;
        this.state.value = value;
        return value;
    }
}
exports.default = Sine;
