"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.processTimerState = exports.parseOptions = exports.getInitialState = void 0;
const now_1 = __importDefault(require("../utils/now"));
const constants_1 = require("../constants");
const getInitialState = (initialTime) => ({
    initialTime,
    isRunning: false,
    iterations: -1,
    prev: 0,
    tickInterval: 0,
    time: initialTime,
    totalElapsed: 0,
});
exports.getInitialState = getInitialState;
const parseOptions = (opts) => {
    return Object.assign({ time: constants_1.SIXTY_FPS }, opts);
};
exports.parseOptions = parseOptions;
const processTimerState = (state, callback) => {
    if (state.iterations === -1) {
        const updates = Object.assign(Object.assign({}, state), { prev: (0, now_1.default)(), iterations: 0 });
        callback(updates);
        return updates;
    }
    const { time, prev, totalElapsed, iterations } = state;
    const tickInterval = (0, now_1.default)() - prev;
    if (tickInterval <= time) {
        return state;
    }
    const updates = Object.assign(Object.assign({}, state), { iterations: iterations + 1, prev: (0, now_1.default)(), tickInterval, totalElapsed: totalElapsed + tickInterval });
    callback(updates);
    return updates;
};
exports.processTimerState = processTimerState;
class Metro {
    constructor(callback, opts) {
        this.stop = () => {
            const { totalElapsed } = this.state;
            this.reset();
            this.clearAsyncHandler();
            return totalElapsed;
        };
        this.reset = () => {
            const { initialTime } = this.state;
            this.state = (0, exports.getInitialState)(initialTime);
        };
        this.setTime = (updatedTime = this.state.time) => {
            const time = Math.max(updatedTime, 0);
            this.state = Object.assign(Object.assign({}, this.state), { time, initialTime: time });
        };
        this.run = () => {
            const { isRunning } = this.state;
            if (isRunning) {
                return;
            }
            this.state = Object.assign(Object.assign({}, this.state), { isRunning: true, prev: (0, now_1.default)() });
            const tick = () => {
                this.clearAsyncHandler();
                (0, exports.processTimerState)(this.state, (updates) => {
                    this.state = updates;
                    this._listeners.forEach((listener) => {
                        listener(this);
                    });
                });
                if (this.state.isRunning) {
                    this.asyncHandler(tick);
                }
            };
            tick();
        };
        const { time } = (0, exports.parseOptions)(opts);
        this.state = (0, exports.getInitialState)(time);
        this._listeners = [callback];
    }
    asyncHandler(callback) {
        this._timerId = setTimeout(callback, constants_1.SIXTY_FPS);
    }
    clearAsyncHandler() {
        clearTimeout(this._timerId);
    }
}
exports.default = Metro;
