"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FORMAT_IDENTIFIERS = void 0;
const constants_1 = require("../constants");
exports.FORMAT_IDENTIFIERS = [
    "fps" /* TimeFormat.FPS */,
    "h" /* TimeFormat.HOURS */,
    "hz" /* TimeFormat.HZ */,
    "ms" /* TimeFormat.MILLISECONDS */,
    "m" /* TimeFormat.MINUTES */,
    "s" /* TimeFormat.SECONDS */,
]
    // Desc length sort so that `ms` matches prior to `m`
    .sort((a, b) => b.length - a.length);
const FORMATTERS = new Map([
    ["fps" /* TimeFormat.FPS */, (val) => constants_1.MS_IN_SECOND / val],
    ["h" /* TimeFormat.HOURS */, (val) => val * constants_1.MS_IN_HOUR],
    ["hz" /* TimeFormat.HZ */, (val) => (1 / val) * constants_1.MS_IN_SECOND],
    ["ms" /* TimeFormat.MILLISECONDS */, (val) => val],
    ["m" /* TimeFormat.MINUTES */, (val) => val * constants_1.MS_IN_MINUTE],
    ["s" /* TimeFormat.SECONDS */, (val) => val * constants_1.MS_IN_SECOND],
]);
const sanitizeStringVal = (val) => val.toLocaleLowerCase().trim();
const parseStringValAndFormat = (val) => {
    for (let i = 0; i < exports.FORMAT_IDENTIFIERS.length; i += 1) {
        const format = exports.FORMAT_IDENTIFIERS[i];
        if (val.includes(format)) {
            return {
                format: exports.FORMAT_IDENTIFIERS[i],
                parsed: Number(val.replace(' ', '').replace(format, '')),
            };
        }
    }
    return {
        format: undefined,
        parsed: undefined,
    };
};
function ms(val, format = "ms" /* TimeFormat.MILLISECONDS */) {
    let parsedValue;
    let parsedFormat = format;
    if (val === null || typeof val === 'undefined') {
        return undefined;
    }
    if (typeof val === 'string') {
        const { parsed: p, format: f } = parseStringValAndFormat(sanitizeStringVal(val));
        if (typeof p !== 'undefined' && format) {
            parsedValue = p;
            parsedFormat = f;
        }
    }
    else {
        parsedValue = val;
    }
    if (Number.isNaN(val)) {
        return undefined;
    }
    const formatter = FORMATTERS.get(parsedFormat);
    if (!formatter) {
        return undefined;
    }
    return formatter(parsedValue);
}
exports.default = ms;
